part of barcodes;

/// The [Code128] is a highly efficient, high-density linear barcode symbology
/// that allows the encoding of alphanumeric data. It is capable of encoding full ASCII character set and extended character sets.
/// This symbology contains the checksum digit for verification and the barcode can also be verified character-by-character for the parity
/// of each data byte.
class Code128 extends Symbology {
  /// Create a [Code128] symbology with the default or required properties.
  ///
  /// The arguments [module] must be non-negative and greater than 0.
  ///
  /// The Code128 symbology encodes the input symbols supported by [Code128A], [Code128B],
  /// [Code128C]. The default symbology type of barcode generator is [Code128].
  ///
  Code128({int module})
      : super(
          module: module,
        ) {
    _code128ACharacterSets = <String>[];

    _code128ACharacterSets.add(' ');
    _code128ACharacterSets.add('!');
    _code128ACharacterSets.add('"');
    _code128ACharacterSets.add('#');
    _code128ACharacterSets.add('\$');
    _code128ACharacterSets.add('%');
    _code128ACharacterSets.add('&');
    _code128ACharacterSets.add('\'');
    _code128ACharacterSets.add('(');
    _code128ACharacterSets.add(')');
    _code128ACharacterSets.add('*');
    _code128ACharacterSets.add('+');
    _code128ACharacterSets.add(',');
    _code128ACharacterSets.add('-');
    _code128ACharacterSets.add('.');
    _code128ACharacterSets.add('/');
    _code128ACharacterSets.add('0');
    _code128ACharacterSets.add('1');
    _code128ACharacterSets.add('2');
    _code128ACharacterSets.add('3');
    _code128ACharacterSets.add('4');
    _code128ACharacterSets.add('5');
    _code128ACharacterSets.add('6');
    _code128ACharacterSets.add('7');
    _code128ACharacterSets.add('8');
    _code128ACharacterSets.add('9');
    _code128ACharacterSets.add(':');
    _code128ACharacterSets.add(';');
    _code128ACharacterSets.add('<');
    _code128ACharacterSets.add('=');
    _code128ACharacterSets.add('>');
    _code128ACharacterSets.add('?');
    _code128ACharacterSets.add('@');
    _code128ACharacterSets.add('A');
    _code128ACharacterSets.add('B');
    _code128ACharacterSets.add('C');
    _code128ACharacterSets.add('D');
    _code128ACharacterSets.add('E');
    _code128ACharacterSets.add('F');
    _code128ACharacterSets.add('G');
    _code128ACharacterSets.add('H');
    _code128ACharacterSets.add('I');
    _code128ACharacterSets.add('J');
    _code128ACharacterSets.add('K');
    _code128ACharacterSets.add('L');
    _code128ACharacterSets.add('M');
    _code128ACharacterSets.add('N');
    _code128ACharacterSets.add('O');
    _code128ACharacterSets.add('P');
    _code128ACharacterSets.add('Q');
    _code128ACharacterSets.add('R');
    _code128ACharacterSets.add('S');
    _code128ACharacterSets.add('T');
    _code128ACharacterSets.add('U');
    _code128ACharacterSets.add('V');
    _code128ACharacterSets.add('W');
    _code128ACharacterSets.add('X');
    _code128ACharacterSets.add('Y');
    _code128ACharacterSets.add('Z');
    _code128ACharacterSets.add('[');
    _code128ACharacterSets.add('\\');
    _code128ACharacterSets.add(']');
    _code128ACharacterSets.add('^');
    _code128ACharacterSets.add('_');
    _code128ACharacterSets.add('\0');
    _code128ACharacterSets.add('\u0001');
    _code128ACharacterSets.add('\u0002');
    _code128ACharacterSets.add('\u0003');
    _code128ACharacterSets.add('\u0004');
    _code128ACharacterSets.add('\u0005');
    _code128ACharacterSets.add('\u0006');
    _code128ACharacterSets.add('\a');
    _code128ACharacterSets.add('\b');
    _code128ACharacterSets.add('\t');
    _code128ACharacterSets.add('\n');
    _code128ACharacterSets.add('\v');
    _code128ACharacterSets.add('\f');
    _code128ACharacterSets.add('\r');
    _code128ACharacterSets.add('\u000e');
    _code128ACharacterSets.add('\u000f');
    _code128ACharacterSets.add('\u0010');
    _code128ACharacterSets.add('\u0011');
    _code128ACharacterSets.add('\u0012');
    _code128ACharacterSets.add('\u0013');
    _code128ACharacterSets.add('\u0014');
    _code128ACharacterSets.add('\u0015');
    _code128ACharacterSets.add('\u0016');
    _code128ACharacterSets.add('\u0017');
    _code128ACharacterSets.add('\u0018');
    _code128ACharacterSets.add('\u0019');
    _code128ACharacterSets.add('\u001a');
    _code128ACharacterSets.add('\u001b');
    _code128ACharacterSets.add('\u001c');
    _code128ACharacterSets.add('\u001d');
    _code128ACharacterSets.add('\u001e');
    _code128ACharacterSets.add('\u001f');
    _code128ACharacterSets.add('ù');
    _code128ACharacterSets.add('ø');
    _code128ACharacterSets.add('û');
    _code128ACharacterSets.add('ö');
    _code128ACharacterSets.add('õ');
    _code128ACharacterSets.add('ú');
    _code128ACharacterSets.add('÷');
    _code128ACharacterSets.add('ü');
    _code128ACharacterSets.add('ý');
    _code128ACharacterSets.add('þ');
    _code128ACharacterSets.add('ÿ');

    _code128BCharacterSets = <String>[];
    _code128BCharacterSets.add(' ');
    _code128BCharacterSets.add('!');
    _code128BCharacterSets.add('"');
    _code128BCharacterSets.add('#');
    _code128BCharacterSets.add('\$');
    _code128BCharacterSets.add('%');
    _code128BCharacterSets.add('&');
    _code128BCharacterSets.add('\'');
    _code128BCharacterSets.add('(');
    _code128BCharacterSets.add(')');
    _code128BCharacterSets.add('*');
    _code128BCharacterSets.add('+');
    _code128BCharacterSets.add(',');
    _code128BCharacterSets.add('-');
    _code128BCharacterSets.add('.');
    _code128BCharacterSets.add('/');
    _code128BCharacterSets.add('0');
    _code128BCharacterSets.add('1');
    _code128BCharacterSets.add('2');
    _code128BCharacterSets.add('3');
    _code128BCharacterSets.add('4');
    _code128BCharacterSets.add('5');
    _code128BCharacterSets.add('6');
    _code128BCharacterSets.add('7');
    _code128BCharacterSets.add('8');
    _code128BCharacterSets.add('9');
    _code128BCharacterSets.add(':');
    _code128BCharacterSets.add(';');
    _code128BCharacterSets.add('<');
    _code128BCharacterSets.add('=');
    _code128BCharacterSets.add('>');
    _code128BCharacterSets.add('?');
    _code128BCharacterSets.add('@');
    _code128BCharacterSets.add('A');
    _code128BCharacterSets.add('B');
    _code128BCharacterSets.add('C');
    _code128BCharacterSets.add('D');
    _code128BCharacterSets.add('E');
    _code128BCharacterSets.add('F');
    _code128BCharacterSets.add('G');
    _code128BCharacterSets.add('H');
    _code128BCharacterSets.add('I');
    _code128BCharacterSets.add('J');
    _code128BCharacterSets.add('K');
    _code128BCharacterSets.add('L');
    _code128BCharacterSets.add('M');
    _code128BCharacterSets.add('N');
    _code128BCharacterSets.add('O');
    _code128BCharacterSets.add('P');
    _code128BCharacterSets.add('Q');
    _code128BCharacterSets.add('R');
    _code128BCharacterSets.add('S');
    _code128BCharacterSets.add('T');
    _code128BCharacterSets.add('U');
    _code128BCharacterSets.add('V');
    _code128BCharacterSets.add('W');
    _code128BCharacterSets.add('X');
    _code128BCharacterSets.add('Y');
    _code128BCharacterSets.add('Z');
    _code128BCharacterSets.add('[');
    _code128BCharacterSets.add('\\');
    _code128BCharacterSets.add(']');
    _code128BCharacterSets.add('^');
    _code128BCharacterSets.add('_');
    _code128BCharacterSets.add('`');
    _code128BCharacterSets.add('a');
    _code128BCharacterSets.add('b');
    _code128BCharacterSets.add('c');
    _code128BCharacterSets.add('d');
    _code128BCharacterSets.add('e');
    _code128BCharacterSets.add('f');
    _code128BCharacterSets.add('g');
    _code128BCharacterSets.add('h');
    _code128BCharacterSets.add('i');
    _code128BCharacterSets.add('j');
    _code128BCharacterSets.add('k');
    _code128BCharacterSets.add('l');
    _code128BCharacterSets.add('m');
    _code128BCharacterSets.add('n');
    _code128BCharacterSets.add('o');
    _code128BCharacterSets.add('p');
    _code128BCharacterSets.add('q');
    _code128BCharacterSets.add('r');
    _code128BCharacterSets.add('s');
    _code128BCharacterSets.add('t');
    _code128BCharacterSets.add('u');
    _code128BCharacterSets.add('v');
    _code128BCharacterSets.add('w');
    _code128BCharacterSets.add('x');
    _code128BCharacterSets.add('y');
    _code128BCharacterSets.add('z');
    _code128BCharacterSets.add('{');
    _code128BCharacterSets.add('|');
    _code128BCharacterSets.add('}');
    _code128BCharacterSets.add('~');
    _code128BCharacterSets.add('\u007f');
    _code128BCharacterSets.add('ù');
    _code128BCharacterSets.add('ø');
    _code128BCharacterSets.add('û');
    _code128BCharacterSets.add('ö');
    _code128BCharacterSets.add('ú');
    _code128BCharacterSets.add('ô');
    _code128BCharacterSets.add('÷');
    _code128BCharacterSets.add('ü');
    _code128BCharacterSets.add('ý');
    _code128BCharacterSets.add('þ');
    _code128BCharacterSets.add('ÿ');

    _code128CCharacterSets = <String>[];
    _code128CCharacterSets.add('0');
    _code128CCharacterSets.add('1');
    _code128CCharacterSets.add('2');
    _code128CCharacterSets.add('3');
    _code128CCharacterSets.add('4');
    _code128CCharacterSets.add('5');
    _code128CCharacterSets.add('6');
    _code128CCharacterSets.add('7');
    _code128CCharacterSets.add('8');
    _code128CCharacterSets.add('9');
    _code128CCharacterSets.add('õ');
    _code128CCharacterSets.add('ô');
    _code128CCharacterSets.add('÷');
    _code128CCharacterSets.add('ü');
    _code128CCharacterSets.add('ý');
    _code128CCharacterSets.add('þ');
    _code128CCharacterSets.add('ÿ');
  }

  /// Represents the supported symbol character of code128A
  List<String> _code128ACharacterSets;

  /// Represents the supported symbol character of code128B
  List<String> _code128BCharacterSets;

  /// Represents the supported symbol character of code128C
  List<String> _code128CCharacterSets;

  /// Represents the start symbol of code128A
  static const int _codeAStartSymbol = 103;

  /// Represents the start symbol for Code128B
  static const int _codeBStartSymbol = 104;

  /// Represents the start symbol for Code128C
  static const int _codeCStartSymbol = 105;

  /// Specifies the value for code128A
  static const int _codeA = 101;

  /// Specifies the value for code128B
  static const int _codeB = 100;

  /// Specifies the value for code128C
  static const int _codeC = 99;

  /// Specifies  the stop symbol
  static const int _codeStopSymbol = 106;

  /// Represents the index value of FNC1 special character
  static const int _codeFNC1 = 102;

  /// Represents the index value of FNC2 special character
  static const int _codeFNC2 = 97;

  /// Represents the index value of FNC3 special character
  static const int _codeFNC3 = 96;

  /// Represents the index value of FNC4A special character
  static const int _codeFNC4A = 101;

  /// Represents the index value of FNC4B special character
  static const int _codeFNC4B = 100;

  /// Represents the FNC1 special character
  static const String _fnc1 = '\u00f1';

  /// Represents the FNC2 special character
  static const String _fnc2 = '\u00f2';

  /// Represents the FNC3 special character
  static const String _fnc3 = '\u00f3';

  /// Represents the FNC4 special character
  static const String _fnc4 = '\u00f4';

  /// Returns the byte value of supported symbol
  List<List<int>> _getCodeValue() {
    return <List<int>>[
      <int>[2, 1, 2, 2, 2, 2],
      <int>[2, 2, 2, 1, 2, 2],
      <int>[2, 2, 2, 2, 2, 1],
      <int>[1, 2, 1, 2, 2, 3],
      <int>[1, 2, 1, 3, 2, 2],
      <int>[1, 3, 1, 2, 2, 2],
      <int>[1, 2, 2, 2, 1, 3],
      <int>[1, 2, 2, 3, 1, 2],
      <int>[1, 3, 2, 2, 1, 2],
      <int>[2, 2, 1, 2, 1, 3],
      <int>[2, 2, 1, 3, 1, 2],
      <int>[2, 3, 1, 2, 1, 2],
      <int>[1, 1, 2, 2, 3, 2],
      <int>[1, 2, 2, 1, 3, 2],
      <int>[1, 2, 2, 2, 3, 1],
      <int>[1, 1, 3, 2, 2, 2],
      <int>[1, 2, 3, 1, 2, 2],
      <int>[1, 2, 3, 2, 2, 1],
      <int>[2, 2, 3, 2, 1, 1],
      <int>[2, 2, 1, 1, 3, 2],
      <int>[2, 2, 1, 2, 3, 1],
      <int>[2, 1, 3, 2, 1, 2],
      <int>[2, 2, 3, 1, 1, 2],
      <int>[3, 1, 2, 1, 3, 1],
      <int>[3, 1, 1, 2, 2, 2],
      <int>[3, 2, 1, 1, 2, 2],
      <int>[3, 2, 1, 2, 2, 1],
      <int>[3, 1, 2, 2, 1, 2],
      <int>[3, 2, 2, 1, 1, 2],
      <int>[3, 2, 2, 2, 1, 1],
      <int>[2, 1, 2, 1, 2, 3],
      <int>[2, 1, 2, 3, 2, 1],
      <int>[2, 3, 2, 1, 2, 1],
      <int>[1, 1, 1, 3, 2, 3],
      <int>[1, 3, 1, 1, 2, 3],
      <int>[1, 3, 1, 3, 2, 1],
      <int>[1, 1, 2, 3, 1, 3],
      <int>[1, 3, 2, 1, 1, 3],
      <int>[1, 3, 2, 3, 1, 1],
      <int>[2, 1, 1, 3, 1, 3],
      <int>[2, 3, 1, 1, 1, 3],
      <int>[2, 3, 1, 3, 1, 1],
      <int>[1, 1, 2, 1, 3, 3],
      <int>[1, 1, 2, 3, 3, 1],
      <int>[1, 3, 2, 1, 3, 1],
      <int>[1, 1, 3, 1, 2, 3],
      <int>[1, 1, 3, 3, 2, 1],
      <int>[1, 3, 3, 1, 2, 1],
      <int>[3, 1, 3, 1, 2, 1],
      <int>[2, 1, 1, 3, 3, 1],
      <int>[2, 3, 1, 1, 3, 1],
      <int>[2, 1, 3, 1, 1, 3],
      <int>[2, 1, 3, 3, 1, 1],
      <int>[2, 1, 3, 1, 3, 1],
      <int>[3, 1, 1, 1, 2, 3],
      <int>[3, 1, 1, 3, 2, 1],
      <int>[3, 3, 1, 1, 2, 1],
      <int>[3, 1, 2, 1, 1, 3],
      <int>[3, 1, 2, 3, 1, 1],
      <int>[3, 3, 2, 1, 1, 1],
      <int>[3, 1, 4, 1, 1, 1],
      <int>[2, 2, 1, 4, 1, 1],
      <int>[4, 3, 1, 1, 1, 1],
      <int>[1, 1, 1, 2, 2, 4],
      <int>[1, 1, 1, 4, 2, 2],
      <int>[1, 2, 1, 1, 2, 4],
      <int>[1, 2, 1, 4, 2, 1],
      <int>[1, 4, 1, 1, 2, 2],
      <int>[1, 4, 1, 2, 2, 1],
      <int>[1, 1, 2, 2, 1, 4],
      <int>[1, 1, 2, 4, 1, 2],
      <int>[1, 2, 2, 1, 1, 4],
      <int>[1, 2, 2, 4, 1, 1],
      <int>[1, 4, 2, 1, 1, 2],
      <int>[1, 4, 2, 2, 1, 1],
      <int>[2, 4, 1, 2, 1, 1],
      <int>[2, 2, 1, 1, 1, 4],
      <int>[4, 1, 3, 1, 1, 1],
      <int>[2, 4, 1, 1, 1, 2],
      <int>[1, 3, 4, 1, 1, 1],
      <int>[1, 1, 1, 2, 4, 2],
      <int>[1, 2, 1, 1, 4, 2],
      <int>[1, 2, 1, 2, 4, 1],
      <int>[1, 1, 4, 2, 1, 2],
      <int>[1, 2, 4, 1, 1, 2],
      <int>[1, 2, 4, 2, 1, 1],
      <int>[4, 1, 1, 2, 1, 2],
      <int>[4, 2, 1, 1, 1, 2],
      <int>[4, 2, 1, 2, 1, 1],
      <int>[2, 1, 2, 1, 4, 1],
      <int>[2, 1, 4, 1, 2, 1],
      <int>[4, 1, 2, 1, 2, 1],
      <int>[1, 1, 1, 1, 4, 3],
      <int>[1, 1, 1, 3, 4, 1],
      <int>[1, 3, 1, 1, 4, 1],
      <int>[1, 1, 4, 1, 1, 3],
      <int>[1, 1, 4, 3, 1, 1],
      <int>[4, 1, 1, 1, 1, 3],
      <int>[4, 1, 1, 3, 1, 1],
      <int>[1, 1, 3, 1, 4, 1],
      <int>[1, 1, 4, 1, 3, 1],
      <int>[3, 1, 1, 1, 4, 1],
      <int>[4, 1, 1, 1, 3, 1],
      <int>[2, 1, 1, 4, 1, 2],
      <int>[2, 1, 1, 2, 1, 4],
      <int>[2, 1, 1, 2, 3, 2],
      <int>[2, 3, 3, 1, 1, 1, 2]
    ];
  }

  @override
  bool _validateInput(String value) {
    for (int i = 0; i < value.length; i++) {
      final int _currentCharacter = value[i].codeUnitAt(0);
      if (_currentCharacter == _fnc1.codeUnitAt(0) ||
          _currentCharacter == _fnc2.codeUnitAt(0) ||
          _currentCharacter == _fnc3.codeUnitAt(0) ||
          _currentCharacter == _fnc4.codeUnitAt(0)) {
        return true;
      } else if (_currentCharacter < 127) {
        return true;
      } else {
        throw 'The provided input cannot be encoded : ' + value[i];
      }
    }
    return false;
  }

  /// Returns the encoded value
  List<List<int>> _getEncodedValue(String value) {
    final List<List<int>> _encodedValue = <List<int>>[];
    final List<List<int>> _bytes = _getCodeValue();
    int _checkDigit = 0;
    int _weightValue = 1;
    int _codeTypeValue = 0;
    int _currentPosition = 0;
    while (_currentPosition < value.length) {
      final int _currentCodeType =
          _validateCode(_currentPosition, _codeTypeValue, value);
      int _currentIndex;
      if (_currentCodeType == _codeTypeValue) {
        final int _currentValue = value[_currentPosition].codeUnitAt(0);
        if (_currentValue == _fnc1.codeUnitAt(0)) {
          _currentIndex = _codeFNC1;
        } else if (_currentValue == _fnc2.codeUnitAt(0)) {
          _currentIndex = _codeFNC2;
        } else if (_currentValue == _fnc3.codeUnitAt(0)) {
          _currentIndex = _codeFNC3;
        } else if (_currentValue == _fnc4.codeUnitAt(0)) {
          if (_currentCodeType == _codeA) {
            _currentIndex = _codeFNC4A;
          } else {
            _currentIndex = _codeFNC4B;
          }
        } else {
          // Calculates the current index value based on code128 type
          if (_currentCodeType == _codeA) {
            _currentIndex =
                value[_currentPosition].codeUnitAt(0) - ' '.codeUnitAt(0);
            if (_currentIndex < 0) {
              _currentIndex += '`'.codeUnitAt(0);
            }
          } else if (_currentCodeType == _codeB) {
            _currentIndex =
                value[_currentPosition].codeUnitAt(0) - ' '.codeUnitAt(0);
          } else {
            _currentIndex = int.parse(
                value.substring(_currentPosition, _currentPosition + 2));
            _currentPosition++;
          }
        }
        _currentPosition++;
      } else {
        if (_codeTypeValue == 0) {
          if (_currentCodeType == _codeA) {
            _currentIndex = _codeAStartSymbol;
          } else if (_currentCodeType == _codeB) {
            _currentIndex = _codeBStartSymbol;
          } else {
            _currentIndex = _codeCStartSymbol;
          }
        } else {
          _currentIndex = _currentCodeType;
        }
        _codeTypeValue = _currentCodeType;
      }
      _encodedValue.add(_bytes[_currentIndex]);
      _checkDigit += _currentIndex * _weightValue;
      if (_currentPosition != 0) {
        _weightValue++;
      }
    }
    _checkDigit %= 103;
    _encodedValue.add(_bytes[_checkDigit]);
    _encodedValue.add(_bytes[_codeStopSymbol]);
    return _encodedValue;
  }

  /// Method to validate the corresponding code set based on the input
  int _validateCode(int _start, int _previousCodeSet, String value) {
    CodeType _codeType = _getCodeType(_start, value);
    if (_codeType == CodeType.singleDigit) {
      if (_previousCodeSet == _codeA) {
        return _codeA;
      }
      return _codeB;
    }

    if (_codeType == CodeType.uncodable) {
      if (_start < value.length) {
        final int _startIndex = value[_start].codeUnitAt(0);
        if (_startIndex < ' '.codeUnitAt(0) ||
            (_previousCodeSet == _codeA &&
                (_startIndex < '`'.codeUnitAt(0) ||
                    (_startIndex >= _fnc1.codeUnitAt(0) &&
                        _startIndex <= _fnc4.codeUnitAt(0))))) {
          return _codeA;
        }
      }
      return _codeB;
    }
    if (_previousCodeSet == _codeA && _codeType == CodeType.fnc1) {
      return _codeA;
    }
    if (_previousCodeSet == _codeC) {
      return _codeC;
    }
    if (_previousCodeSet == _codeB) {
      if (_codeType == CodeType.fnc1) {
        return _codeB;
      }
      _codeType = _getCodeType(_start + 2, value);
      if (_codeType == CodeType.uncodable ||
          _codeType == CodeType.singleDigit) {
        return _codeB;
      }
      if (_codeType == CodeType.fnc1) {
        _codeType = _getCodeType(_start + 3, value);
        if (_codeType == CodeType.doubleDigit) {
          return this is Code128C ? _codeC : _codeB;
        } else {
          return _codeB;
        }
      }
      int _currentIndex = _start + 4;
      while (_getCodeType(_currentIndex, value) == CodeType.doubleDigit) {
        _currentIndex += 2;
      }
      if (_codeType == CodeType.singleDigit) {
        return _codeB;
      }
      return this is Code128B ? _codeB : _codeC;
    }

    if (_codeType == CodeType.fnc1) {
      _codeType = _getCodeType(_start + 1, value);
    }

    if (_codeType == CodeType.doubleDigit) {
      return this is Code128B ? _codeB : _codeC;
    }
    return _codeB;
  }

  /// Returns the code type based on the input
  CodeType _getCodeType(int _startIndex, String value) {
    final int _length = value.length;
    if (_startIndex >= _length) {
      return CodeType.uncodable;
    }
    if (String.fromCharCode(value[_startIndex].codeUnitAt(0)) ==
        String.fromCharCode(_fnc1.codeUnitAt(0))) {
      return CodeType.fnc1;
    }
    if (value[_startIndex].codeUnitAt(0) < '0'.codeUnitAt(0) ||
        value[_startIndex].codeUnitAt(0) > '9'.codeUnitAt(0)) {
      return CodeType.uncodable;
    }

    if (_startIndex + 1 >= _length) {
      return CodeType.singleDigit;
    }

    if (value[_startIndex + 1].codeUnitAt(0) < '0'.codeUnitAt(0) ||
        value[_startIndex + 1].codeUnitAt(0) > '9'.codeUnitAt(0)) {
      return CodeType.singleDigit;
    }

    return CodeType.doubleDigit;
  }

  @override
  void _renderBarcode(
      Canvas canvas,
      Size size,
      Offset offset,
      String value,
      Color foregroundColor,
      TextStyle textStyle,
      double textSpacing,
      TextAlign textAlign,
      bool showValue) {
    final Paint _paint = _getBarPaint(foregroundColor);
    final List<List<int>> _encodedValue = _getEncodedValue(value);
    final int _totalBarLength = _getTotalBarLength(_encodedValue);
    double _left = module == null
        ? offset.dx
        : _calculateLeftPosition(
            _totalBarLength, module, size.width, offset.dx);
    double _ratio = 0;
    if (module != null) {
      _ratio = module.toDouble();
    } else {
      // Calculates the bar length based on number of individual bar codes
      final int singleModule = (size.width ~/ _totalBarLength).toInt();
      _ratio = singleModule.toDouble();
      final double _leftPadding = (size.width - (_totalBarLength * _ratio)) / 2;
      _left += _leftPadding;
    }
    _left = _left.roundToDouble();
    for (int i = 0; i < _encodedValue.length; i++) {
      bool _canDraw = true;
      final List<int> _currentIndex = _encodedValue[i];
      for (int j = 0; j < _currentIndex.length; j++) {
        final int _currentValue = _currentIndex[j];
        // Draws the bar code based on the current value
        for (int k = 0; k < _currentValue; k++) {
          if (_canDraw) {
            final Rect _individualBarRect = Rect.fromLTRB(
                _left, offset.dy, _left + _ratio, offset.dy + size.height);
            canvas.drawRect(_individualBarRect, _paint);
          }
          _left += _ratio;
        }
        _canDraw = !_canDraw;
      }
    }
    if (showValue)
      _drawText(canvas, offset, size, value, textStyle, textSpacing, textAlign);
  }

  /// Calculate total bar length from give input value
  int _getTotalBarLength(List<List<int>> _encodedValue) {
    int _length = 0;
    for (int i = 0; i < _encodedValue.length; i++) {
      final List<int> _currentValue = _encodedValue[i];
      for (int j = 0; j < _currentValue.length; j++) {
        _length += _currentValue[j];
      }
    }
    return _length;
  }
}
