part of barcodes;

/// Represents the module value
class _ModuleValue {
  /// Creates the module value
  _ModuleValue() {
    _isBlack = false;
    _isFilled = false;
    _isPDP = false;
  }

  /// Specifies whether the the dot is black
  bool _isBlack;

  /// Specifies whether the the dot is already filled
  bool _isFilled;

  /// Specifies whether the the dot is PDP
  bool _isPDP;
}

/// Represents the QR Code value
class _QRCodeValue {
  /// Creates the QR code value
  _QRCodeValue({this.qrCodeVersion, this.errorCorrectionLevel}) {
    _noOfDataCodeWord = _obtainNumberOfDataCodeWord();
    _noOfErrorCorrectionCodeWord = _obtainNumberOfErrorCorrectionCodeWords();
    _noOfErrorCorrectionBlocks = _obtainNumberOfErrorCorrectionBlocks();
    _formatInformation = _obtainFormatInformation();
    _versionInformation = _obtainVersionInformation();
  }

  /// Specifies the code version
  final QRCodeVersion qrCodeVersion;

  /// Specifies the error correction level
  final ErrorCorrectionLevel errorCorrectionLevel;

  /// Specifies the number of data code word
  int _noOfDataCodeWord;

  /// Specifies the number of error correction code word
  int _noOfErrorCorrectionCodeWord;

  /// Specifies the number of error correction blocks
  List<int> _noOfErrorCorrectionBlocks;

  /// Specifies the format information
  List<int> _formatInformation;

  /// Specifies the version information
  List<int> _versionInformation;

  /// Specifies the nuber of error correction code words
  static const List<int> _noOfErrorCorrectionCodeWords = <int>[
    7,
    10,
    13,
    17,
    10,
    16,
    22,
    28,
    15,
    26,
    36,
    44,
    20,
    36,
    52,
    64,
    26,
    48,
    72,
    88,
    36,
    64,
    96,
    112,
    40,
    72,
    108,
    130,
    48,
    88,
    132,
    156,
    60,
    110,
    160,
    192,
    72,
    130,
    192,
    224,
    80,
    150,
    224,
    264,
    96,
    176,
    260,
    308,
    104,
    198,
    288,
    352,
    120,
    216,
    320,
    384,
    132,
    240,
    360,
    432,
    144,
    280,
    408,
    480,
    168,
    308,
    448,
    532,
    180,
    338,
    504,
    588,
    196,
    364,
    546,
    650,
    224,
    416,
    600,
    700,
    224,
    442,
    644,
    750,
    252,
    476,
    690,
    816,
    270,
    504,
    750,
    900,
    300,
    560,
    810,
    960,
    312,
    588,
    870,
    1050,
    336,
    644,
    952,
    1110,
    360,
    700,
    1020,
    1200,
    390,
    728,
    1050,
    1260,
    420,
    784,
    1140,
    1350,
    450,
    812,
    1200,
    1440,
    480,
    868,
    1290,
    1530,
    510,
    924,
    1350,
    1620,
    540,
    980,
    1440,
    1710,
    570,
    1036,
    1530,
    1800,
    570,
    1064,
    1590,
    1890,
    600,
    1120,
    1680,
    1980,
    630,
    1204,
    1770,
    2100,
    660,
    1260,
    1860,
    2220,
    720,
    1316,
    1950,
    2310,
    750,
    1372,
    2040,
    2430
  ];

  /// Specifies the value of numeric data capacity of low error correction level
  static const List<int> _numericDataCapacityLow = <int>[
    41,
    77,
    127,
    187,
    255,
    322,
    370,
    461,
    552,
    652,
    772,
    883,
    1022,
    1101,
    1250,
    1408,
    1548,
    1725,
    1903,
    2061,
    2232,
    2409,
    2620,
    2812,
    3057,
    3283,
    3517,
    3669,
    3909,
    4158,
    4417,
    4686,
    4965,
    5253,
    5529,
    5836,
    6153,
    6479,
    6743,
    7089,
  ];

  /// Specifies the value of numeric data capacity of medium error correction level
  static const List<int> _numericDataCapacityMedium = <int>[
    34,
    63,
    101,
    149,
    202,
    255,
    293,
    365,
    432,
    513,
    604,
    691,
    796,
    871,
    991,
    1082,
    1212,
    1346,
    1500,
    1600,
    1708,
    1872,
    2059,
    2188,
    2395,
    2544,
    2701,
    2857,
    3035,
    3289,
    3486,
    3693,
    3909,
    4134,
    4343,
    4588,
    4775,
    5039,
    5313,
    5596,
  ];

  /// Specifies the value of numeric data capacity of quartile error correction level
  static const List<int> _numericDataCapacityQuartile = <int>[
    27,
    48,
    77,
    111,
    144,
    178,
    207,
    259,
    312,
    364,
    427,
    489,
    580,
    621,
    703,
    775,
    876,
    948,
    1063,
    1159,
    1224,
    1358,
    1468,
    1588,
    1718,
    1804,
    1933,
    2085,
    2181,
    2358,
    2473,
    2670,
    2805,
    2949,
    3081,
    3244,
    3417,
    3599,
    3791,
    3993,
  ];

  /// Specifies the value of numeric data capacity of high error correction level
  static const List<int> _numericDataCapacityHigh = <int>[
    17,
    34,
    58,
    82,
    106,
    139,
    154,
    202,
    235,
    288,
    331,
    374,
    427,
    468,
    530,
    602,
    674,
    746,
    813,
    919,
    969,
    1056,
    1108,
    1228,
    1286,
    1425,
    1501,
    1581,
    1677,
    1782,
    1897,
    2022,
    2157,
    2301,
    2361,
    2524,
    2625,
    2735,
    2927,
    3057
  ];

  /// Specifies the value of alpha numeric data capacity of low error correction level
  static const List<int> _alphaNumericDataCapacityLow = <int>[
    25,
    47,
    77,
    114,
    154,
    195,
    224,
    279,
    335,
    395,
    468,
    535,
    619,
    667,
    758,
    854,
    938,
    1046,
    1153,
    1249,
    1352,
    1460,
    1588,
    1704,
    1853,
    1990,
    2132,
    2223,
    2369,
    2520,
    2677,
    2840,
    3009,
    3183,
    3351,
    3537,
    3729,
    3927,
    4087,
    4296
  ];

  /// Specifies the value of alpha numeric data capacity of high error correction level
  static const List<int> _alphaNumericDataCapacityMedium = <int>[
    20,
    38,
    61,
    90,
    122,
    154,
    178,
    221,
    262,
    311,
    366,
    419,
    483,
    528,
    600,
    656,
    734,
    816,
    909,
    970,
    1035,
    1134,
    1248,
    1326,
    1451,
    1542,
    1637,
    1732,
    1839,
    1994,
    2113,
    2238,
    2369,
    2506,
    2632,
    2780,
    2894,
    3054,
    3220,
    3391
  ];

  /// Specifies the value of alpha numeric data capacity of quartile error correction level
  static const List<int> _alphaNumericDataCapacityQuartile = <int>[
    16,
    29,
    47,
    67,
    87,
    108,
    125,
    157,
    189,
    221,
    259,
    296,
    352,
    376,
    426,
    470,
    531,
    574,
    644,
    702,
    742,
    823,
    890,
    963,
    1041,
    1094,
    1172,
    1263,
    1322,
    1429,
    1499,
    1618,
    1700,
    1787,
    1867,
    1966,
    2071,
    2181,
    2298,
    2420
  ];

  /// Specifies the value of alpha numeric data capacity of high error correction level
  static const List<int> _alphaNumericDataCapacityHigh = <int>[
    10,
    20,
    35,
    50,
    64,
    84,
    93,
    122,
    143,
    174,
    200,
    227,
    259,
    283,
    321,
    365,
    408,
    452,
    493,
    557,
    587,
    640,
    672,
    744,
    779,
    864,
    910,
    958,
    1016,
    1080,
    1150,
    1226,
    1307,
    1394,
    1431,
    1530,
    1591,
    1658,
    1774,
    1852
  ];

  /// Specifies the value of binary data capacity of low error correction level
  static const List<int> _binaryDataCapacityLow = <int>[
    17,
    32,
    53,
    78,
    106,
    134,
    154,
    192,
    230,
    271,
    321,
    367,
    425,
    458,
    520,
    586,
    644,
    718,
    792,
    858,
    929,
    1003,
    1091,
    1171,
    1273,
    1367,
    1465,
    1528,
    1628,
    1732,
    1840,
    1952,
    2068,
    2188,
    2303,
    2431,
    2563,
    2699,
    2809,
    2953
  ];

  /// Specifies the value of binary data capacity of medium error correction level
  static const List<int> _binaryDataCapacityMedium = <int>[
    14,
    26,
    42,
    62,
    84,
    106,
    122,
    152,
    180,
    213,
    251,
    287,
    331,
    362,
    412,
    450,
    504,
    560,
    624,
    666,
    711,
    779,
    857,
    911,
    997,
    1059,
    1125,
    1190,
    1264,
    1370,
    1452,
    1538,
    1628,
    1722,
    1809,
    1911,
    1989,
    2099,
    2213,
    2331
  ];

  /// Specifies the value of binary data capacity of quartile error correction level
  static const List<int> _binaryDataCapacityQuartile = <int>[
    11,
    20,
    32,
    46,
    60,
    74,
    86,
    108,
    130,
    151,
    177,
    203,
    241,
    258,
    292,
    322,
    364,
    394,
    442,
    482,
    509,
    565,
    611,
    661,
    715,
    751,
    805,
    868,
    908,
    982,
    1030,
    1112,
    1168,
    1228,
    1283,
    1351,
    1423,
    1499,
    1579,
    1663
  ];

  /// Specifies the value of binary data capacity of high error correction level
  static const List<int> _binaryDataCapacityHigh = <int>[
    7,
    14,
    24,
    34,
    44,
    58,
    64,
    84,
    98,
    119,
    137,
    155,
    177,
    194,
    220,
    250,
    280,
    310,
    338,
    382,
    403,
    439,
    461,
    511,
    535,
    593,
    625,
    658,
    698,
    742,
    790,
    842,
    898,
    958,
    983,
    1051,
    1093,
    1139,
    1219,
    1273
  ];

  /// REturns the alpha numeric
  int _getAlphaNumericValues(String _value) {
    int _valueInInt = 0;
    switch (_value) {
      case '0':
        _valueInInt = 0;
        break;
      case '1':
        _valueInInt = 1;
        break;
      case '2':
        _valueInInt = 2;
        break;
      case '3':
        _valueInInt = 3;
        break;
      case '4':
        _valueInInt = 4;
        break;
      case '5':
        _valueInInt = 5;
        break;
      case '6':
        _valueInInt = 6;
        break;
      case '7':
        _valueInInt = 7;
        break;
      case '8':
        _valueInInt = 8;
        break;
      case '9':
        _valueInInt = 9;
        break;
      case 'A':
        _valueInInt = 10;
        break;
      case 'B':
        _valueInInt = 11;
        break;
      case 'C':
        _valueInInt = 12;
        break;
      case 'D':
        _valueInInt = 13;
        break;
      case 'E':
        _valueInInt = 14;
        break;
      case 'F':
        _valueInInt = 15;
        break;
      case 'G':
        _valueInInt = 16;
        break;
      case 'H':
        _valueInInt = 17;
        break;
      case 'I':
        _valueInInt = 18;
        break;
      case 'J':
        _valueInInt = 19;
        break;
      case 'K':
        _valueInInt = 20;
        break;
      case 'L':
        _valueInInt = 21;
        break;
      case 'M':
        _valueInInt = 22;
        break;
      case 'N':
        _valueInInt = 23;
        break;
      case 'O':
        _valueInInt = 24;
        break;
      case 'P':
        _valueInInt = 25;
        break;
      case 'Q':
        _valueInInt = 26;
        break;
      case 'R':
        _valueInInt = 27;
        break;
      case 'S':
        _valueInInt = 28;
        break;
      case 'T':
        _valueInInt = 29;
        break;
      case 'U':
        _valueInInt = 30;
        break;
      case 'V':
        _valueInInt = 31;
        break;
      case 'W':
        _valueInInt = 32;
        break;
      case 'X':
        _valueInInt = 33;
        break;
      case 'Y':
        _valueInInt = 34;
        break;
      case 'Z':
        _valueInInt = 35;
        break;
      case ' ':
        _valueInInt = 36;
        break;
      case '\$':
        _valueInInt = 37;
        break;
      case '%':
        _valueInInt = 38;
        break;
      case '*':
        _valueInInt = 39;
        break;
      case '+':
        _valueInInt = 40;
        break;
      case '-':
        _valueInInt = 41;
        break;
      case '.':
        _valueInInt = 42;
        break;
      case '/':
        _valueInInt = 43;
        break;
      case ':':
        _valueInInt = 44;
        break;
      default:
        _valueInInt = null;
        break;
    }

    return _valueInInt;
  }

  ///Calculates the number of data code word
  int _obtainNumberOfDataCodeWord() {
    int _countOfDataCodeWord = 0;
    switch (qrCodeVersion) {
      case QRCodeVersion.version01:
        _countOfDataCodeWord = _setDataCodeWordForVersion01();
        break;
      case QRCodeVersion.version02:
        _countOfDataCodeWord = _setDataCodeWordForVersion02();
        break;
      case QRCodeVersion.version03:
        _countOfDataCodeWord = _setDataCodeWordForVersion03();
        break;
      case QRCodeVersion.version04:
        _countOfDataCodeWord = _setDataCodeWordForVersion04();
        break;
      case QRCodeVersion.version05:
        _countOfDataCodeWord = _setDataCodeWordForVersion05();
        break;
      case QRCodeVersion.version06:
        _countOfDataCodeWord = _setDataCodeWordForVersion06();
        break;
      case QRCodeVersion.version07:
        _countOfDataCodeWord = _setDataCodeWordForVersion07();
        break;
      case QRCodeVersion.version08:
        _countOfDataCodeWord = _setDataCodeWordForVersion08();
        break;
      case QRCodeVersion.version09:
        _countOfDataCodeWord = _setDataCodeWordForVersion09();
        break;
      case QRCodeVersion.version10:
        _countOfDataCodeWord = _setDataCodeWordForVersion10();
        break;
      case QRCodeVersion.version11:
        _countOfDataCodeWord = _setDataCodeWordForVersion11();
        break;
      case QRCodeVersion.version12:
        _countOfDataCodeWord = _setDataCodeWordForVersion12();
        break;
      case QRCodeVersion.version13:
        _countOfDataCodeWord = _setDataCodeWordForVersion13();
        break;
      case QRCodeVersion.version14:
        _countOfDataCodeWord = _setDataCodeWordForVersion14();
        break;
      case QRCodeVersion.version15:
        _countOfDataCodeWord = _setDataCodeWordForVersion15();
        break;
      case QRCodeVersion.version16:
        _countOfDataCodeWord = _setDataCodeWordForVersion16();
        break;
      case QRCodeVersion.version17:
        _countOfDataCodeWord = _setDataCodeWordForVersion17();
        break;
      case QRCodeVersion.version18:
        _countOfDataCodeWord = _setDataCodeWordForVersion18();
        break;
      case QRCodeVersion.version19:
        _countOfDataCodeWord = _setDataCodeWordForVersion19();
        break;
      case QRCodeVersion.version20:
        _countOfDataCodeWord = _setDataCodeWordForVersion20();
        break;
      case QRCodeVersion.version21:
        _countOfDataCodeWord = _setDataCodeWordForVersion21();
        break;
      case QRCodeVersion.version22:
        _countOfDataCodeWord = _setDataCodeWordForVersion22();
        break;
      case QRCodeVersion.version23:
        _countOfDataCodeWord = _setDataCodeWordForVersion23();
        break;
      case QRCodeVersion.version24:
        _countOfDataCodeWord = _setDataCodeWordForVersion24();
        break;
      case QRCodeVersion.version25:
        _countOfDataCodeWord = _setDataCodeWordForVersion25();
        break;
      case QRCodeVersion.version26:
        _countOfDataCodeWord = _setDataCodeWordForVersion26();
        break;
      case QRCodeVersion.version27:
        _countOfDataCodeWord = _setDataCodeWordForVersion27();
        break;
      case QRCodeVersion.version28:
        _countOfDataCodeWord = _setDataCodeWordForVersion28();
        break;
      case QRCodeVersion.version29:
        _countOfDataCodeWord = _setDataCodeWordForVersion29();
        break;
      case QRCodeVersion.version30:
        _countOfDataCodeWord = _setDataCodeWordForVersion30();
        break;
      case QRCodeVersion.version31:
        _countOfDataCodeWord = _setDataCodeWordForVersion31();
        break;
      case QRCodeVersion.version32:
        _countOfDataCodeWord = _setDataCodeWordForVersion32();
        break;
      case QRCodeVersion.version33:
        _countOfDataCodeWord = _setDataCodeWordForVersion33();
        break;
      case QRCodeVersion.version34:
        _countOfDataCodeWord = _setDataCodeWordForVersion34();
        break;
      case QRCodeVersion.version35:
        _countOfDataCodeWord = _setDataCodeWordForVersion35();
        break;
      case QRCodeVersion.version36:
        _countOfDataCodeWord = _setDataCodeWordForVersion36();
        break;
      case QRCodeVersion.version37:
        _countOfDataCodeWord = _setDataCodeWordForVersion37();
        break;
      case QRCodeVersion.version38:
        _countOfDataCodeWord = _setDataCodeWordForVersion38();
        break;
      case QRCodeVersion.version39:
        _countOfDataCodeWord = _setDataCodeWordForVersion39();
        break;
      case QRCodeVersion.version40:
        _countOfDataCodeWord = _setDataCodeWordForVersion40();
        break;
      case QRCodeVersion.auto:
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 1
  int _setDataCodeWordForVersion01() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 19;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 16;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 13;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 9;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 2
  int _setDataCodeWordForVersion02() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 34;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 28;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 22;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 16;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 3
  int _setDataCodeWordForVersion03() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 55;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 44;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 34;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 26;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 4
  int _setDataCodeWordForVersion04() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 80;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 64;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 48;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 36;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 5
  int _setDataCodeWordForVersion05() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 108;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 86;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 62;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 46;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 6
  int _setDataCodeWordForVersion06() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 136;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 108;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 76;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 60;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 7
  int _setDataCodeWordForVersion07() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 156;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 124;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 88;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 66;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 8
  int _setDataCodeWordForVersion08() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 194;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 154;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 110;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 86;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 9
  int _setDataCodeWordForVersion09() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 232;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 182;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 132;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 100;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 10
  int _setDataCodeWordForVersion10() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 274;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 216;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 154;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 122;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 11
  int _setDataCodeWordForVersion11() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 324;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 254;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 180;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 140;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 12
  int _setDataCodeWordForVersion12() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 370;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 290;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 206;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 158;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 13
  int _setDataCodeWordForVersion13() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 428;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 334;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 244;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 180;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 14
  int _setDataCodeWordForVersion14() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 461;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 365;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 261;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 197;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 15
  int _setDataCodeWordForVersion15() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 523;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 415;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 295;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 223;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 16
  int _setDataCodeWordForVersion16() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 589;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 453;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 325;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 253;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 17
  int _setDataCodeWordForVersion17() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 647;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 507;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 367;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 283;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 18
  int _setDataCodeWordForVersion18() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 721;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 563;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 397;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 313;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 19
  int _setDataCodeWordForVersion19() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 795;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 627;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 445;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 341;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 20
  int _setDataCodeWordForVersion20() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 861;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 669;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 485;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 385;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 21
  int _setDataCodeWordForVersion21() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 932;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 714;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 512;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 406;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 22
  int _setDataCodeWordForVersion22() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 1006;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 782;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 568;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 442;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 23
  int _setDataCodeWordForVersion23() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 1094;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 860;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 614;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 464;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 24
  int _setDataCodeWordForVersion24() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 1174;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 914;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 664;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 514;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 25
  int _setDataCodeWordForVersion25() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 1276;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 1000;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 718;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 538;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 26
  int _setDataCodeWordForVersion26() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 1370;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 1062;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 754;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 596;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 27
  int _setDataCodeWordForVersion27() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 1468;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 1128;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 808;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 628;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 28
  int _setDataCodeWordForVersion28() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 1531;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 1193;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 871;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 661;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 29
  int _setDataCodeWordForVersion29() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 1631;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 1267;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 911;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 701;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 30
  int _setDataCodeWordForVersion30() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 1735;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 1373;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 985;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 745;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 31
  int _setDataCodeWordForVersion31() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 1843;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 1455;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 1033;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 793;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 32
  int _setDataCodeWordForVersion32() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 1955;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 1541;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 1115;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 845;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 33
  int _setDataCodeWordForVersion33() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 2071;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 1631;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 1171;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 901;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 34
  int _setDataCodeWordForVersion34() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 2191;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 1725;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 1231;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 961;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 35
  int _setDataCodeWordForVersion35() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 2306;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 1812;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 1286;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 986;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 36
  int _setDataCodeWordForVersion36() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 2434;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 1914;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 1354;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 1054;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 37
  int _setDataCodeWordForVersion37() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 2566;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 1992;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 1426;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 1096;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 38
  int _setDataCodeWordForVersion38() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 2702;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 2102;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 1502;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 1142;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 39
  int _setDataCodeWordForVersion39() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 2812;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 2216;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 1582;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 1222;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Specifies the data code word for version 40
  int _setDataCodeWordForVersion40() {
    int _countOfDataCodeWord = 0;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _countOfDataCodeWord = 2956;
        break;
      case ErrorCorrectionLevel.medium:
        _countOfDataCodeWord = 2334;
        break;
      case ErrorCorrectionLevel.quartile:
        _countOfDataCodeWord = 1666;
        break;
      case ErrorCorrectionLevel.high:
        _countOfDataCodeWord = 1276;
        break;
    }

    return _countOfDataCodeWord;
  }

  /// Calculates the number of error corrcetion code words
  int _obtainNumberOfErrorCorrectionCodeWords() {
    int _index = (_getVersionNumber(qrCodeVersion) - 1) * 4;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _index += 0;
        break;
      case ErrorCorrectionLevel.medium:
        _index += 1;
        break;
      case ErrorCorrectionLevel.quartile:
        _index += 2;
        break;
      case ErrorCorrectionLevel.high:
        _index += 3;
        break;
    }

    return _noOfErrorCorrectionCodeWords[_index];
  }

  /// Specifies the number of error correction blocks
  List<int> _obtainNumberOfErrorCorrectionBlocks() {
    List<int> _numberOfErrorCorrectionBlocks;
    final int _version = _getVersionNumber(qrCodeVersion);
    switch (_version) {
      case 1:
      case 2:
        _numberOfErrorCorrectionBlocks = <int>[1];
        break;
      case 3:
        _numberOfErrorCorrectionBlocks = _getErrorCorrectionBlocksForVersion3();
        break;
      case 4:
        _numberOfErrorCorrectionBlocks = _getErrorCorrectionBlocksForVersion4();
        break;
      case 5:
        _numberOfErrorCorrectionBlocks = _getErrorCorrectionBlocksForVersion5();
        break;
      case 6:
        _numberOfErrorCorrectionBlocks = _getErrorCorrectionBlocksForVersion6();
        break;
      case 7:
        _numberOfErrorCorrectionBlocks = _getErrorCorrectionBlocksForVersion7();
        break;
      case 8:
        _numberOfErrorCorrectionBlocks = _getErrorCorrectionBlocksForVersion8();
        break;
      case 9:
        _numberOfErrorCorrectionBlocks = _getErrorCorrectionBlocksForVersion9();
        break;
      case 10:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion10();
        break;
      case 11:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion11();
        break;
      case 12:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion12();
        break;
      case 13:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion13();
        break;
      case 14:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion14();
        break;
      case 15:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion15();
        break;
      case 16:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion16();
        break;
      case 17:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion17();
        break;
      case 18:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion18();
        break;
      case 19:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion19();
        break;
      case 20:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion20();
        break;
      case 21:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion21();
        break;
      case 22:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion22();
        break;
      case 23:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion23();
        break;
      case 24:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion24();
        break;
      case 25:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion25();
        break;
      case 26:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion26();
        break;
      case 27:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion27();
        break;
      case 28:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion28();
        break;
      case 29:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion29();
        break;
      case 30:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion30();
        break;
      case 31:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion31();
        break;
      case 32:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion32();
        break;
      case 33:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion33();
        break;
      case 34:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion34();
        break;
      case 35:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion35();
        break;
      case 36:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion36();
        break;
      case 37:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion37();
        break;
      case 38:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion38();
        break;
      case 39:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion39();
        break;
      case 40:
        _numberOfErrorCorrectionBlocks =
            _getErrorCorrectionBlocksForVersion40();
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 3
  List<int> _getErrorCorrectionBlocksForVersion3() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[1];
        break;
      case ErrorCorrectionLevel.quartile:
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[2];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 4
  List<int> _getErrorCorrectionBlocksForVersion4() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[1];
        break;
      case ErrorCorrectionLevel.medium:
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[2];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[4];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 5
  List<int> _getErrorCorrectionBlocksForVersion5() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[1];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[2];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[2, 33, 15, 2, 34, 16];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[2, 33, 11, 2, 34, 12];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 6
  List<int> _getErrorCorrectionBlocksForVersion6() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[2];
        break;
      case ErrorCorrectionLevel.medium:
      case ErrorCorrectionLevel.quartile:
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[4];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 7
  List<int> _getErrorCorrectionBlocksForVersion7() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[2];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[4];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[2, 32, 14, 4, 33, 15];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[4, 39, 13, 1, 40, 14];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 8
  List<int> _getErrorCorrectionBlocksForVersion8() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[2];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[2, 60, 38, 2, 61, 39];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[4, 40, 18, 2, 41, 19];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[4, 40, 14, 2, 41, 15];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 9
  List<int> _getErrorCorrectionBlocksForVersion9() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[2];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[3, 58, 36, 2, 59, 37];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[4, 36, 16, 4, 37, 17];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[4, 36, 12, 4, 37, 13];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 10
  List<int> _getErrorCorrectionBlocksForVersion10() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[2, 86, 68, 2, 87, 69];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[4, 69, 43, 1, 70, 44];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[6, 43, 19, 2, 44, 20];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[6, 43, 15, 2, 44, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version11
  List<int> _getErrorCorrectionBlocksForVersion11() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[4];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[1, 80, 50, 4, 81, 51];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[4, 50, 22, 4, 51, 23];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[3, 36, 12, 8, 37, 13];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 12
  List<int> _getErrorCorrectionBlocksForVersion12() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[2, 116, 92, 2, 117, 93];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[6, 58, 36, 2, 59, 37];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[4, 46, 20, 6, 47, 21];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[7, 42, 14, 4, 43, 15];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 13
  List<int> _getErrorCorrectionBlocksForVersion13() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[4];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[8, 59, 37, 1, 60, 38];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[8, 44, 20, 4, 45, 21];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[12, 33, 11, 4, 34, 12];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 14
  List<int> _getErrorCorrectionBlocksForVersion14() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[3, 145, 115, 1, 146, 116];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[4, 64, 40, 5, 65, 41];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[11, 36, 16, 5, 37, 17];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[11, 36, 12, 5, 37, 13];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 15
  List<int> _getErrorCorrectionBlocksForVersion15() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[5, 109, 87, 1, 110, 88];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[5, 65, 41, 5, 66, 42];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[5, 54, 24, 7, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[11, 36, 12, 7, 37, 13];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 16
  List<int> _getErrorCorrectionBlocksForVersion16() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[5, 112, 98, 1, 123, 99];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[7, 73, 45, 3, 74, 46];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[15, 43, 19, 2, 44, 20];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[3, 45, 15, 13, 46, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 17
  List<int> _getErrorCorrectionBlocksForVersion17() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[1, 135, 107, 5, 136, 108];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[10, 74, 46, 1, 75, 47];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[1, 50, 22, 15, 51, 23];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[2, 42, 14, 17, 43, 15];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 18
  List<int> _getErrorCorrectionBlocksForVersion18() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[5, 150, 120, 1, 151, 121];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[9, 69, 43, 4, 70, 44];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[17, 50, 22, 1, 51, 23];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[2, 42, 14, 19, 43, 15];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 19
  List<int> _getErrorCorrectionBlocksForVersion19() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[3, 141, 113, 4, 142, 114];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[3, 70, 44, 11, 71, 45];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[17, 47, 21, 4, 48, 22];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[9, 39, 13, 16, 40, 14];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the error correction blocks for version 20
  List<int> _getErrorCorrectionBlocksForVersion20() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[3, 135, 107, 5, 136, 108];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[3, 67, 41, 13, 68, 42];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[15, 54, 24, 5, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[15, 43, 15, 10, 44, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 21
  List<int> _getErrorCorrectionBlocksForVersion21() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[4, 144, 116, 4, 145, 117];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[17];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[17, 50, 22, 6, 51, 23];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[19, 46, 16, 6, 47, 17];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 22
  List<int> _getErrorCorrectionBlocksForVersion22() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[2, 139, 111, 7, 140, 112];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[17];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[7, 54, 24, 16, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[34];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 23
  List<int> _getErrorCorrectionBlocksForVersion23() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[4, 151, 121, 5, 152, 122];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[4, 75, 47, 14, 76, 48];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[11, 54, 24, 14, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[16, 45, 15, 14, 46, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 24
  List<int> _getErrorCorrectionBlocksForVersion24() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[6, 147, 117, 4, 148, 118];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[6, 73, 45, 14, 74, 46];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[11, 54, 24, 16, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[30, 46, 16, 2, 47, 17];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 25
  List<int> _getErrorCorrectionBlocksForVersion25() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[8, 132, 106, 4, 133, 107];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[8, 75, 47, 13, 76, 48];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[7, 54, 24, 22, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[22, 45, 15, 13, 46, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 26
  List<int> _getErrorCorrectionBlocksForVersion26() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[10, 142, 114, 2, 143, 115];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[19, 74, 46, 4, 75, 47];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[28, 50, 22, 6, 51, 23];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[33, 46, 16, 4, 47, 17];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 27
  List<int> _getErrorCorrectionBlocksForVersion27() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[8, 152, 122, 4, 153, 123];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[22, 73, 45, 3, 74, 46];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[8, 53, 23, 26, 54, 24];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[12, 45, 15, 28, 46, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 28
  List<int> _getErrorCorrectionBlocksForVersion28() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[3, 147, 117, 10, 148, 118];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[3, 73, 45, 23, 74, 46];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[4, 54, 24, 31, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[11, 45, 15, 31, 46, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 29
  List<int> _getErrorCorrectionBlocksForVersion29() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[7, 146, 116, 7, 147, 117];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[21, 73, 45, 7, 74, 46];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[1, 53, 23, 37, 54, 24];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[19, 45, 15, 26, 46, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 30
  List<int> _getErrorCorrectionBlocksForVersion30() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[5, 145, 115, 10, 146, 116];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[19, 75, 47, 10, 76, 48];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[15, 54, 24, 25, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[23, 45, 15, 25, 46, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 31
  List<int> _getErrorCorrectionBlocksForVersion31() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[13, 145, 115, 3, 146, 116];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[2, 74, 46, 29, 75, 47];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[42, 54, 24, 1, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[23, 45, 15, 28, 46, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 32
  List<int> _getErrorCorrectionBlocksForVersion32() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[17];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[10, 74, 46, 23, 75, 47];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[10, 54, 24, 35, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[19, 45, 15, 35, 46, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 33
  List<int> _getErrorCorrectionBlocksForVersion33() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[17, 145, 115, 1, 146, 116];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[14, 74, 46, 21, 75, 47];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[29, 54, 24, 19, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[11, 45, 15, 46, 46, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 34
  List<int> _getErrorCorrectionBlocksForVersion34() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[13, 145, 115, 6, 146, 116];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[14, 74, 46, 23, 75, 47];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[44, 54, 24, 7, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[59, 46, 16, 1, 47, 17];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 35
  List<int> _getErrorCorrectionBlocksForVersion35() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[12, 151, 121, 7, 152, 122];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[12, 75, 47, 26, 76, 48];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[39, 54, 24, 14, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[22, 45, 15, 41, 46, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 36
  List<int> _getErrorCorrectionBlocksForVersion36() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[6, 151, 121, 14, 152, 122];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[6, 75, 47, 34, 76, 48];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[46, 54, 24, 10, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[2, 45, 15, 64, 46, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 37
  List<int> _getErrorCorrectionBlocksForVersion37() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[17, 152, 122, 4, 153, 123];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[29, 74, 46, 14, 75, 47];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[49, 54, 24, 10, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[24, 45, 15, 46, 46, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 38
  List<int> _getErrorCorrectionBlocksForVersion38() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[4, 152, 122, 18, 153, 123];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[13, 74, 46, 32, 75, 47];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[48, 54, 24, 14, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[42, 45, 15, 32, 46, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 39
  List<int> _getErrorCorrectionBlocksForVersion39() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[20, 147, 117, 4, 148, 118];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[40, 75, 47, 7, 76, 48];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[43, 54, 24, 22, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[10, 45, 15, 67, 46, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  ///Specifies the error correction blocks for version 40
  List<int> _getErrorCorrectionBlocksForVersion40() {
    List<int> _numberOfErrorCorrectionBlocks;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _numberOfErrorCorrectionBlocks = <int>[19, 148, 118, 6, 149, 119];
        break;
      case ErrorCorrectionLevel.medium:
        _numberOfErrorCorrectionBlocks = <int>[18, 75, 47, 31, 76, 48];
        break;
      case ErrorCorrectionLevel.quartile:
        _numberOfErrorCorrectionBlocks = <int>[34, 54, 24, 34, 55, 25];
        break;
      case ErrorCorrectionLevel.high:
        _numberOfErrorCorrectionBlocks = <int>[20, 45, 15, 61, 46, 16];
        break;
    }

    return _numberOfErrorCorrectionBlocks;
  }

  /// Specifies the format information
  List<int> _obtainFormatInformation() {
    List<int> _formatInformation;
    switch (errorCorrectionLevel) {
      case ErrorCorrectionLevel.low:
        _formatInformation = <int>[1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1];
        break;
      case ErrorCorrectionLevel.medium:
        _formatInformation = <int>[1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 1, 0, 1];
        break;
      case ErrorCorrectionLevel.quartile:
        _formatInformation = <int>[0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0];
        break;
      case ErrorCorrectionLevel.high:
        _formatInformation = <int>[0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 0];
        break;
    }

    return _formatInformation;
  }

  /// Specifies the version information
  List<int> _obtainVersionInformation() {
    List<int> _versionInformation;
    final int _version = _getVersionNumber(qrCodeVersion);
    switch (_version) {
      case 7:
        _versionInformation = <int>[
          0,
          0,
          1,
          0,
          1,
          0,
          0,
          1,
          0,
          0,
          1,
          1,
          1,
          1,
          1,
          0,
          0,
          0
        ];
        break;
      case 8:
        _versionInformation = <int>[
          0,
          0,
          1,
          1,
          1,
          1,
          0,
          1,
          1,
          0,
          1,
          0,
          0,
          0,
          0,
          1,
          0,
          0
        ];
        break;
      case 9:
        _versionInformation = <int>[
          1,
          0,
          0,
          1,
          1,
          0,
          0,
          1,
          0,
          1,
          0,
          1,
          1,
          0,
          0,
          1,
          0,
          0
        ];
        break;
      case 10:
        _versionInformation = <int>[
          1,
          1,
          0,
          0,
          1,
          0,
          1,
          1,
          0,
          0,
          1,
          0,
          0,
          1,
          0,
          1,
          0,
          0
        ];
        break;
      case 11:
        _versionInformation = <int>[
          0,
          1,
          1,
          0,
          1,
          1,
          1,
          1,
          1,
          1,
          0,
          1,
          1,
          1,
          0,
          1,
          0,
          0
        ];
        break;
      case 12:
        _versionInformation = <int>[
          0,
          1,
          0,
          0,
          0,
          1,
          1,
          0,
          1,
          1,
          1,
          0,
          0,
          0,
          1,
          1,
          0,
          0
        ];
        break;
      case 13:
        _versionInformation = <int>[
          1,
          1,
          1,
          0,
          0,
          0,
          1,
          0,
          0,
          0,
          0,
          1,
          1,
          0,
          1,
          1,
          0,
          0
        ];
        break;
      case 14:
        _versionInformation = <int>[
          1,
          0,
          1,
          1,
          0,
          0,
          0,
          0,
          0,
          1,
          1,
          0,
          0,
          1,
          1,
          1,
          0,
          0
        ];
        break;
      case 15:
        _versionInformation = <int>[
          0,
          0,
          0,
          1,
          0,
          1,
          0,
          0,
          1,
          0,
          0,
          1,
          1,
          1,
          1,
          1,
          0,
          0
        ];
        break;
      case 16:
        _versionInformation = <int>[
          0,
          0,
          0,
          1,
          1,
          1,
          1,
          0,
          1,
          1,
          0,
          1,
          0,
          0,
          0,
          0,
          1,
          0
        ];
        break;
      case 17:
        _versionInformation = <int>[
          1,
          0,
          1,
          1,
          1,
          0,
          1,
          0,
          0,
          0,
          1,
          0,
          1,
          0,
          0,
          0,
          1,
          0
        ];
        break;
      case 18:
        _versionInformation = <int>[
          1,
          1,
          1,
          0,
          1,
          0,
          0,
          0,
          0,
          1,
          0,
          1,
          0,
          1,
          0,
          0,
          1,
          0
        ];
        break;
      case 19:
        _versionInformation = <int>[
          0,
          1,
          0,
          0,
          1,
          1,
          0,
          0,
          1,
          0,
          1,
          0,
          1,
          1,
          0,
          0,
          1,
          0
        ];
        break;
      case 20:
        _versionInformation = <int>[
          0,
          1,
          1,
          0,
          0,
          1,
          0,
          1,
          1,
          0,
          0,
          1,
          0,
          0,
          1,
          0,
          1,
          0
        ];
        break;
      case 21:
        _versionInformation = <int>[
          1,
          1,
          0,
          0,
          0,
          0,
          0,
          1,
          0,
          1,
          1,
          0,
          1,
          0,
          1,
          0,
          1,
          0
        ];
        break;
      case 22:
        _versionInformation = <int>[
          1,
          0,
          0,
          1,
          0,
          0,
          1,
          1,
          0,
          0,
          0,
          1,
          0,
          1,
          1,
          0,
          1,
          0
        ];
        break;
      case 23:
        _versionInformation = <int>[
          0,
          0,
          1,
          1,
          0,
          1,
          1,
          1,
          1,
          1,
          1,
          0,
          1,
          1,
          1,
          0,
          1,
          0
        ];
        break;
      case 24:
        _versionInformation = <int>[
          0,
          0,
          1,
          0,
          0,
          0,
          1,
          1,
          0,
          1,
          1,
          1,
          0,
          0,
          0,
          1,
          1,
          0
        ];
        break;
      case 25:
        _versionInformation = <int>[
          1,
          0,
          0,
          0,
          0,
          1,
          1,
          1,
          1,
          0,
          0,
          0,
          1,
          0,
          0,
          1,
          1,
          0
        ];
        break;
      case 26:
        _versionInformation = <int>[
          1,
          1,
          0,
          1,
          0,
          1,
          0,
          1,
          1,
          1,
          1,
          1,
          0,
          1,
          0,
          1,
          1,
          0
        ];
        break;
      case 27:
        _versionInformation = <int>[
          0,
          1,
          1,
          1,
          0,
          0,
          0,
          1,
          0,
          0,
          0,
          0,
          1,
          1,
          0,
          1,
          1,
          0
        ];
        break;
      case 28:
        _versionInformation = <int>[
          0,
          1,
          0,
          1,
          1,
          0,
          0,
          0,
          0,
          0,
          1,
          1,
          0,
          0,
          1,
          1,
          1,
          0
        ];
        break;
      case 29:
        _versionInformation = <int>[
          1,
          1,
          1,
          1,
          1,
          1,
          0,
          0,
          1,
          1,
          0,
          0,
          1,
          0,
          1,
          1,
          1,
          0
        ];
        break;
      case 30:
        _versionInformation = <int>[
          1,
          0,
          1,
          0,
          1,
          1,
          1,
          0,
          1,
          0,
          1,
          1,
          0,
          1,
          1,
          1,
          1,
          0
        ];
        break;
      case 31:
        _versionInformation = <int>[
          0,
          0,
          0,
          0,
          1,
          0,
          1,
          0,
          0,
          1,
          0,
          0,
          1,
          1,
          1,
          1,
          1,
          0
        ];
        break;
      case 32:
        _versionInformation = <int>[
          1,
          0,
          1,
          0,
          1,
          0,
          1,
          1,
          1,
          0,
          0,
          1,
          0,
          0,
          0,
          0,
          0,
          1
        ];
        break;
      case 33:
        _versionInformation = <int>[
          0,
          0,
          0,
          0,
          1,
          1,
          1,
          1,
          0,
          1,
          1,
          0,
          1,
          0,
          0,
          0,
          0,
          1
        ];
        break;
      case 34:
        _versionInformation = <int>[
          0,
          1,
          0,
          1,
          1,
          1,
          0,
          1,
          0,
          0,
          0,
          1,
          0,
          1,
          0,
          0,
          0,
          1
        ];
        break;
      case 35:
        _versionInformation = <int>[
          1,
          1,
          1,
          1,
          1,
          0,
          0,
          1,
          1,
          1,
          1,
          0,
          1,
          1,
          0,
          0,
          0,
          1
        ];
        break;
      case 36:
        _versionInformation = <int>[
          1,
          1,
          0,
          1,
          0,
          0,
          0,
          0,
          1,
          1,
          0,
          1,
          0,
          0,
          1,
          0,
          0,
          1
        ];
        break;
      case 37:
        _versionInformation = <int>[
          0,
          1,
          1,
          1,
          0,
          1,
          0,
          0,
          0,
          0,
          1,
          0,
          1,
          0,
          1,
          0,
          0,
          1
        ];
        break;
      case 38:
        _versionInformation = <int>[
          0,
          0,
          1,
          0,
          0,
          1,
          1,
          0,
          0,
          1,
          0,
          1,
          0,
          1,
          1,
          0,
          0,
          1
        ];
        break;
      case 39:
        _versionInformation = <int>[
          1,
          0,
          0,
          0,
          0,
          0,
          1,
          0,
          1,
          0,
          1,
          0,
          1,
          1,
          1,
          0,
          0,
          1
        ];
        break;
      case 40:
        _versionInformation = <int>[
          1,
          0,
          0,
          1,
          0,
          1,
          1,
          0,
          0,
          0,
          1,
          1,
          0,
          0,
          0,
          1,
          0,
          1
        ];
        break;
    }

    return _versionInformation;
  }

  /// Returns the numeric data capacity
  static int _getNumericDataCapacity(
      ErrorCorrectionLevel _level, QRCodeVersion _codeVersion) {
    List<int> _capacity;
    switch (_level) {
      case ErrorCorrectionLevel.low:
        _capacity = _numericDataCapacityLow;
        break;
      case ErrorCorrectionLevel.medium:
        _capacity = _numericDataCapacityMedium;
        break;
      case ErrorCorrectionLevel.quartile:
        _capacity = _numericDataCapacityQuartile;
        break;
      case ErrorCorrectionLevel.high:
        _capacity = _numericDataCapacityHigh;
        break;
    }

    final int _version = _getVersionNumber(_codeVersion);
    return _capacity[_version - 1];
  }

  /// Specifies the alpha numeric data capcity
  static int _getAlphaNumericDataCapacity(
      ErrorCorrectionLevel _level, QRCodeVersion _codeVersion) {
    List<int> _capacity;
    switch (_level) {
      case ErrorCorrectionLevel.low:
        _capacity = _alphaNumericDataCapacityLow;
        break;
      case ErrorCorrectionLevel.medium:
        _capacity = _alphaNumericDataCapacityMedium;
        break;
      case ErrorCorrectionLevel.quartile:
        _capacity = _alphaNumericDataCapacityQuartile;
        break;
      case ErrorCorrectionLevel.high:
        _capacity = _alphaNumericDataCapacityHigh;
        break;
    }

    final int _version = _getVersionNumber(_codeVersion);
    return _capacity[_version - 1];
  }

  /// Specifies the bunary data capacity
  static int _getBinaryDataCapacity(
      ErrorCorrectionLevel _level, QRCodeVersion _codeVersion) {
    List<int> _capacity;
    switch (_level) {
      case ErrorCorrectionLevel.low:
        _capacity = _binaryDataCapacityLow;
        break;
      case ErrorCorrectionLevel.medium:
        _capacity = _binaryDataCapacityMedium;
        break;
      case ErrorCorrectionLevel.quartile:
        _capacity = _binaryDataCapacityQuartile;
        break;
      case ErrorCorrectionLevel.high:
        _capacity = _binaryDataCapacityHigh;
        break;
    }

    final int _version = _getVersionNumber(_codeVersion);
    return _capacity[_version - 1];
  }
}
